@@ -17,11 +17,12 @@ from PyQt5.QtWidgets import (
 
 from feeluown.utils import aio
 from feeluown.utils.dispatch import Signal
-from feeluown.library import ModelState, ModelFlags
-from feeluown.models import ModelExistence
+from feeluown.library import ModelState, ModelFlags, MediaFlags
 
 from feeluown.gui.mimedata import ModelMimeData
-from feeluown.gui.helpers import ItemViewNoScrollMixin, ReaderFetchMoreMixin
+from feeluown.gui.helpers import (
+    ItemViewNoScrollMixin, ReaderFetchMoreMixin, painter_save
+)
 
 
 logger = logging.getLogger(__name__)
@@ -182,7 +183,9 @@ class SongListDelegate(QStyledItemDelegate):
                     self.play_btn_pressed = True
                 if event.type() == QEvent.MouseButtonRelease:
                     if self.play_btn_pressed is True:
-                        self.parent().play_song_needed.emit(index.data(Qt.UserRole))
+                        parent = self.parent()
+                        assert isinstance(parent, SongListView)
+                        parent.play_song_needed.emit(index.data(Qt.UserRole))
             if event.type() == QEvent.MouseButtonRelease:
                 self.play_btn_pressed = False
         return super().editorEvent(event, model, option, index)
@@ -199,8 +202,7 @@ class SongListView(ItemViewNoScrollMixin, QListView):
     play_song_needed = pyqtSignal([object])
 
     def __init__(self, parent=None, **kwargs):
-        super().__init__(parent, **kwargs)
-        QListView.__init__(self, parent)
+        super().__init__(parent=parent, **kwargs)
 
         self.delegate = SongListDelegate(self)
         self.setItemDelegate(self.delegate)
@@ -249,12 +251,8 @@ class BaseSongsTableModel(QAbstractTableModel):
         # If song's state is `not_exists` or `cant_upgrade`, the album and
         # artist columns are disabled.
         incomplete = False
-        if ModelFlags.v2 & song.meta.flags:
-            if song.state in (ModelState.not_exists, ModelState.cant_upgrade):
-                incomplete = True
-        else:
-            if song and song.exists == ModelExistence.no:
-                incomplete = True
+        if ModelState(song.state) in (ModelState.not_exists, ModelState.cant_upgrade):
+            incomplete = True
         if incomplete:
             if index.column() != Column.song:
                 flags = no_item_flags
@@ -280,7 +278,9 @@ class BaseSongsTableModel(QAbstractTableModel):
                 # we set height to 25 since the header can be short under macOS.
                 # HELP: set height to fixed value manually is not so elegant
                 height = 25
-                w = self.columns_config.get_width(section, self.parent().width())
+                parent = self.parent()
+                assert isinstance(parent, QWidget)
+                w = self.columns_config.get_width(section, parent.width())
                 return QSize(w, height)
         else:
             if role == Qt.DisplayRole:
@@ -470,6 +470,9 @@ class SongsTableDelegate(QStyledItemDelegate):
         painter.setRenderHint(QPainter.Antialiasing)
         hovered = index.row() == self.row_hovered
 
+        if index.column() == Column.song:
+            self.paint_vip_tag(painter, option, index)
+
         # Draw play button on Column.index when the row is hovered.
         if hovered and index.column() == Column.index:
             painter.save()
@@ -511,6 +514,31 @@ class SongsTableDelegate(QStyledItemDelegate):
             painter.drawRect(option.rect)
             painter.restore()
 
+    def paint_vip_tag(self, painter, option, index):
+        song = index.data(Qt.UserRole)
+        if (ModelFlags.normal in ModelFlags(song.meta.flags) and
+                MediaFlags.vip in MediaFlags(song.media_flags)):
+            with painter_save(painter):
+                fm = option.fontMetrics
+                title = index.data(Qt.DisplayRole)
+                title_rect = fm.boundingRect(title)
+                if title_rect.width() < option.rect.width():
+                    # Tested on (KDE and macOS):
+                    #   when font size is 7px, text width~>16 & height~>10
+                    font = option.font
+                    font.setPixelSize(7)
+                    painter.setFont(font)
+                    text_width, text_height = 16, 10
+                    y = option.rect.y() + (option.rect.height() - text_height) // 2
+                    # NOTE(cosven): On macOS, the acture width of text is large than
+                    # title_rect.width(), which is also true on KDE. This is decided
+                    # by QStyle. +10px works well on macOS, and it also works well
+                    # on KDE (actually, from local test, 7px is enough for KDE).
+                    x = option.rect.x() + title_rect.width() + 10
+                    text_rect = QRect(x, y, text_width, text_height)
+                    painter.drawRoundedRect(text_rect, 3, 3)
+                    painter.drawText(text_rect, Qt.AlignCenter, 'VIP')
+
     def sizeHint(self, option, index):
         """set proper width for each column
 
@@ -520,8 +548,10 @@ class SongsTableDelegate(QStyledItemDelegate):
         """
         if index.isValid() and self.parent() is not None:
             # The way getting the sourceModel seems a little strange.
+            parent = self.parent()
+            assert isinstance(parent, QWidget)
             w = index.model().sourceModel().columns_config.get_width(
-                index.column(), self.parent().width())
+                index.column(), parent.width())
             h = option.rect.height()
             return QSize(w, h)
         return super().sizeHint(option, index)
@@ -534,7 +564,9 @@ class SongsTableDelegate(QStyledItemDelegate):
                 self.pressed_cell = cell
             elif etype == QEvent.MouseButtonRelease:
                 if cell == self.pressed_cell and cell[1] == Column.index:
-                    self.parent().play_song_needed.emit(index.data(Qt.UserRole))
+                    parent = self.parent()
+                    assert isinstance(parent, SongsTableView)
+                    parent.play_song_needed.emit(index.data(Qt.UserRole))
                 self.pressed_cell = None
 
         return super().editorEvent(event, model, option, index)
@@ -555,8 +587,7 @@ class SongsTableView(ItemViewNoScrollMixin, QTableView):
     row_hovered = pyqtSignal([object])  # None when not hovered, row id when hovered.
 
     def __init__(self, app, parent=None):
-        super().__init__(parent)
-        QTableView.__init__(self, parent)
+        super().__init__(parent=parent)
 
         self._app = app
 
@@ -612,9 +643,14 @@ class SongsTableView(ItemViewNoScrollMixin, QTableView):
             hide_columns = [Column.source]
             columns_config.set_width_ratio(Column.artist, 0.2)
             columns_config.set_width_ratio(Column.album, 0.3)
-        self.model().sourceModel().update_columns_config(columns_config)
 
-        for i in range(0, self.model().columnCount()):
+        model = self.model()
+        assert isinstance(model, SongFilterProxyModel)
+        source = model.sourceModel()
+        assert isinstance(source, SongsTableModel)
+        source.update_columns_config(columns_config)
+
+        for i in range(0, model.columnCount()):
             if i in hide_columns:
                 self.hideColumn(i)
             else:
@@ -636,12 +672,13 @@ class SongsTableView(ItemViewNoScrollMixin, QTableView):
         menu.addAction(add_to_playlist_action)
 
         # remove song action
-        if self.remove_song_func is not None:
-            remove_song_action = QAction('移除歌曲', menu)
-            remove_song_action.triggered.connect(
-                lambda: self._remove_by_indexes(indexes))
-            menu.addSeparator()
-            menu.addAction(remove_song_action)
+        remove_song_action = QAction('移除歌曲', menu)
+        remove_song_action.triggered.connect(
+            lambda: self._remove_by_indexes(indexes))
+        menu.addSeparator()
+        menu.addAction(remove_song_action)
+        if self.remove_song_func is None:
+            remove_song_action.setDisabled(True)
 
         model = self.model()
         models = [model.data(index, Qt.UserRole) for index in indexes]
@@ -678,6 +715,7 @@ class SongsTableView(ItemViewNoScrollMixin, QTableView):
             if song not in songs_to_remove:
                 songs_to_remove.append(song)
         for song in songs_to_remove:
+            assert callable(self.remove_song_func)
             self.remove_song_func(song)
 
     def viewportEvent(self, event):
